﻿package com.callystro.engine {
	import com.callystro.resource.ResourceManager;
	import com.callystro.sound.SoundPlayer;
	import flash.display.MovieClip;

	/**
	 * ...
	 * @author Callystro12
	 * version 2.0
	 */
	public class IntroPlayer {
		private static var voIndex:Number;
		private static var totalNoInstruction:Number;
		private static var onComplete:Function;
		private static var voXml:XML;
		private static var arrowMC:MovieClip;
		private static var instruction:XMLList;
		private static var animationMC:MovieClip;
		private static var lastVOComplete:Boolean;
		private static var playInstructionEnabled:Boolean;
		private static var nextStopFrame:int;

		public function IntroPlayer() {

		}

		public static function playInstructionVO(level:int, onComplete:Function=null, arrowMC:MovieClip=null, animationMC:MovieClip=null):Function {
			voXml = ResourceManager.getInstance().getXML("IntroAnimation");
			instruction = voXml.Level.(@no == String(level)).instruction;
			voIndex = -1;
			totalNoInstruction = instruction.length();
			trace("Total-No-Vos:" + totalNoInstruction);
			lastVOComplete = true;
			playInstructionEnabled = true;
			nextStopFrame = 1;
			IntroPlayer.onComplete = onComplete;
			IntroPlayer.voXml = voXml;
			IntroPlayer.arrowMC = arrowMC;
			IntroPlayer.animationMC = animationMC;
			playNextVo();

			return update;
		}

		public static function update():void {
			if (playInstructionEnabled) {
				if (animationMC != null) {
					if (animationMC.currentFrame < nextStopFrame) {
						animationMC.nextFrame();
					}
					if (lastVOComplete && animationMC.currentFrame == nextStopFrame) {
						playNextVo();
					}
				} else {
					if (lastVOComplete) {
						playNextVo();
					}
				}
			}
		}

		private static function playNextVo():void {
			if (lastVOComplete) {
				voIndex++;
				if (voIndex < totalNoInstruction) {
					playNextSound();
					lastVOComplete = false;
					if (animationMC != null) {
						animationMC.gotoAndStop(instruction[voIndex].@startingFrameNO);
						nextStopFrame = instruction[voIndex].@endingFrameNO;
					}
					if (arrowMC != null) {
						arrowMC.visible = (instruction[voIndex].@showArrow == "true");
						arrowMC.x = Number(instruction[voIndex].@arrowX);
						arrowMC.y = Number(instruction[voIndex].@arrowY);
						arrowMC.scaleX = instruction[voIndex].@arrowScaleX;
						arrowMC.scaleY = instruction[voIndex].@arrowScaleX;
						arrowMC.rotation = instruction[voIndex].@rotation;
					}

				} else {
					arrowMC = null;
					if (onComplete != null) {
						var onCmt:Function = onComplete;
						onComplete = null;
						onCmt();
					}
				}
			}
		}

		private static function lastVOCompleted():void {
			lastVOComplete = true;
			if (arrowMC != null)
				arrowMC.visible = false;
		}

		static private function playNextSound():void {
			var soundKey:String = instruction[voIndex].@soundKey;
			trace("Playing sound : " + soundKey);
			SoundPlayer.play("IntroPlayerCurrentSound", ResourceManager.getInstance().getSound(soundKey), lastVOCompleted, 1, 0);
		}

		public static function stopInstruction(callOnComplete:Boolean=false):void {
			SoundPlayer.stop("IntroPlayerCurrentSound");
			playInstructionEnabled = false;
			if (arrowMC != null)
				arrowMC.visible = false;
			if (callOnComplete && onComplete != null) {
				onComplete();
			}
		}

		public static function stopIntro(callOnComplete:Boolean=false):void {
			stopInstruction(callOnComplete);
		}

		static public function pause():void {
			try {
				playInstructionEnabled = false;
				if (!lastVOComplete)
					SoundPlayer.pause("IntroPlayerCurrentSound");
			} catch (e:Error) {
				trace("IntroPlayer Pausing sound error : " + e);
			}
		}

		static public function resume():void {
			try {
				playInstructionEnabled = true;
				if (!lastVOComplete)
					SoundPlayer.resume("IntroPlayerCurrentSound");
			} catch (e:Error) {
				trace("IntroPlayer Resuming sound error : " + e);
			}
		}
	}

}