﻿package com.callystro.resource{

	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.events.Event;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.media.Sound;
	import flash.text.StyleSheet;
	import flash.text.TextFormat;
	import flash.utils.Dictionary;
	
	public class ResourceManager implements IEventDispatcher {
		//Events Dispatched by Resource Manager
		public static const INITIALIZED:String="ResourceManager.Initialized";
		public static const ERROR:String="ResourceManager.Error";

		private var definitionFileLoader:URLLoader;

		private var definitionFileUrl:String;

		private var definitionFile:XML;
		
		private var additionalRelativePath:String = "";
		
		private static var resourceMangers:Dictionary = new Dictionary();

		private var resourceConatiners:Array;

		private var initialized:Boolean=false;

		private var multipleResourceLoader:MultipleResourceLoader;
		
		private var levels:Array;
		
		public function ResourceManager() {
			dispatcher = new EventDispatcher();
		}

		//Event Dispatcher Code
		private var dispatcher:EventDispatcher = new EventDispatcher();
		public function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void {
			dispatcher.addEventListener(type, listener, useCapture, priority);
		}

		public function dispatchEvent(evt:Event):Boolean {
			return dispatcher.dispatchEvent(evt);
		}

		public function hasEventListener(type:String):Boolean {
			return dispatcher.hasEventListener(type);
		}

		public function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void {
			dispatcher.removeEventListener(type, listener, useCapture);
		}

		public function willTrigger(type:String):Boolean {
			return dispatcher.willTrigger(type);
		}

		public function initialize(resourceDefinitionFileUrl:String):void {
				dispose();
				definitionFileUrl = additionalRelativePath + resourceDefinitionFileUrl;
				definitionFileLoader = new URLLoader();
				definitionFileLoader.addEventListener(Event.COMPLETE,definitionFileLoadComplete);
				definitionFileLoader.addEventListener(IOErrorEvent.IO_ERROR,definitionFileLoadError);
				definitionFileLoader.load(new URLRequest(definitionFileUrl));
		}

		protected function definitionFileLoadComplete(e:Event):void {
			definitionFileLoader.removeEventListener(IOErrorEvent.IO_ERROR,definitionFileLoadError);
			definitionFileLoader.removeEventListener(Event.COMPLETE,definitionFileLoadComplete);
			definitionFileLoader = null;
			definitionFile=new XML(e.target.data);
			dispatchEvent(new Event(INITIALIZED));
		}

		protected function definitionFileLoadError(e:Event):void {
			definitionFileLoader.removeEventListener(IOErrorEvent.IO_ERROR,definitionFileLoadError);
			definitionFileLoader.removeEventListener(Event.COMPLETE,definitionFileLoadComplete);
			definitionFileLoader = null;
			dispatchEvent(new Event(ERROR));
		}

		public function load(levels:Array = null):void {
			var resourceContainer:ResourceContainer;
			var i:int=0;
			multipleResourceLoader = new MultipleResourceLoader();
			if (resourceConatiners==null) {
				resourceConatiners = new Array();
				if (definitionFile.CommonResources.length()>0) {
					resourceContainer=new ResourceContainer(definitionFile.CommonResources, 0 ,additionalRelativePath,true);
					resourceContainer.populateResourceLoader(multipleResourceLoader);
					resourceConatiners.push(resourceContainer);
				}
			} else {
				for (i = 0; i < resourceConatiners.length; i++) {
					resourceContainer=resourceConatiners[i];
					if (! resourceContainer.keepInMemory) {
						trace("Removing resourceContainer"+resourceContainer);
						resourceContainer.dispose();
						resourceConatiners.splice(i,1);
						i--;
					}
				}
			}
			if (levels!=null&&levels.length>0) {
				for (i = 0; i < levels.length; i++) {
					for (var j:int = 0; j <  definitionFile.LevelResources.length(); j++) {
						if (Number(definitionFile.LevelResources[j].@no)==Number(levels[i])) {
							resourceContainer=new ResourceContainer(definitionFile.LevelResources[j], int(Number(definitionFile.LevelResources[j].@no)),additionalRelativePath);
							resourceContainer.populateResourceLoader(multipleResourceLoader);
							resourceConatiners.push(resourceContainer);
						}
					}
				}
			}
			levels = new Array();
			for(i = 0; i < resourceConatiners.length; i++)
			{
				if((resourceConatiners[i] as ResourceContainer).levelNumber != 0)
				{
					levels.push((resourceConatiners[i] as ResourceContainer).levelNumber);
				}
			}
			multipleResourceLoader.addEventListener(Event.COMPLETE,onLoadComplete);
			multipleResourceLoader.addEventListener(ProgressEvent.PROGRESS,dispatchEvent);
			multipleResourceLoader.addEventListener(IOErrorEvent.IO_ERROR,onLoadError);
			multipleResourceLoader.load();
		}
		
		public function getLoadedLevels():Array
		{
			return levels;
		}
		
		public static function getInstance(id:String = null):ResourceManager {
			if(id == null || id == "")
			{
				id = "Default";
			}
			if (resourceMangers[id] == undefined || resourceMangers[id] == null) {
				
				var resourceManger:ResourceManager = new ResourceManager();
				resourceMangers[id] = resourceManger;
			}
			return resourceMangers[id];
		}
		
		public function setAdditionalRelativePath(relativePath:String):void
		{
			additionalRelativePath = relativePath;
		}

		protected function onLoadComplete(e:Event):void {
			multipleResourceLoader.removeEventListener(Event.COMPLETE,onLoadComplete);
			multipleResourceLoader.removeEventListener(ProgressEvent.PROGRESS,dispatchEvent);
			multipleResourceLoader.removeEventListener(IOErrorEvent.IO_ERROR,onLoadError);
			multipleResourceLoader.dispose();
			multipleResourceLoader=null;
			dispatchEvent(e);
		}

		protected function onLoadError(e:Event):void {
			multipleResourceLoader.removeEventListener(Event.COMPLETE,onLoadComplete);
			multipleResourceLoader.removeEventListener(ProgressEvent.PROGRESS,dispatchEvent);
			multipleResourceLoader.removeEventListener(IOErrorEvent.IO_ERROR,onLoadError);
			multipleResourceLoader.dispose();
			multipleResourceLoader=null;
			dispatchEvent(e);
		}

		public function getText(key:String, textArguments:Array = null):String {
			var returnValue:String=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getText(key, textArguments);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getSound(key:String):Sound {
			var returnValue:Sound=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getSound(key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getSWF(key:String):Object {
			var returnValue:Object=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getSWF(key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getStyleSheet(key:String):StyleSheet {
			var returnValue:StyleSheet=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getStyleSheet(key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getFontClass(key:String):Class {
			var returnValue:Class=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getFontClass(key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getStyle(styleName:String,key:String):Object {
			var returnValue:Object=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getStyle(styleName,key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}

		public function getTextFormat(styleName:String,key:String):TextFormat {
			var returnValue:TextFormat=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getTextFormat(styleName,key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}
		
		public function getXML(key:String):XML {
			var returnValue:XML=null;
			for (var i:int = 0; i < resourceConatiners.length; i++) {
				returnValue = (resourceConatiners[i] as ResourceContainer).getXML(key);
				// Found the required resource
				if (returnValue!=null) {
					break;
				}
			}
			return returnValue;
		}
		
		public function dispose():void {
			var resourceContainer:ResourceContainer;
			var i:int = 0;
			if(resourceConatiners != null){
				for (i = 0; i < resourceConatiners.length; i++) {
					resourceContainer=resourceConatiners[i];
					resourceContainer.dispose();
				}
			}
			resourceContainer = null;
			resourceConatiners = null;
			if(multipleResourceLoader != null){
				multipleResourceLoader.removeEventListener(Event.COMPLETE,onLoadComplete);
				multipleResourceLoader.removeEventListener(ProgressEvent.PROGRESS,dispatchEvent);
				multipleResourceLoader.removeEventListener(IOErrorEvent.IO_ERROR,onLoadError);
				multipleResourceLoader.dispose();
				multipleResourceLoader = null;
			}
			if(definitionFileLoader != null){
				definitionFileLoader.removeEventListener(IOErrorEvent.IO_ERROR,definitionFileLoadError);
				definitionFileLoader.removeEventListener(Event.COMPLETE,definitionFileLoadComplete);
				definitionFileLoader = null;
			}
			definitionFile = null;
		}
	}

}