﻿package com.callystro.sound {
	
	import flash.events.EventDispatcher;
	import flash.events.Event;
	import flash.media.Sound;
	import flash.media.SoundTransform;
	import flash.media.SoundChannel;
	
	public class SoundInstance extends EventDispatcher{
		
		public static const SOUND_PLAYBACK_STARTED:String = "SoundPlaybackStarted";
		public static const SOUND_COMPLETE:String = "SoundComplete";
		public static const SOUND_PAUSED:String = "SoundPaused";
		public static const SOUND_RESUMED:String = "SoundResumed";
		public static const SOUND_VOLUME_CHANGED:String = "SoundVolumeChanged";
		public static const SOUND_VOLUME_RESTORED:String = "SoundVolumeRestored";
		public static const SOUND_STOPPED:String = "SoundStopped";
		
		public var key:String;
		public var sound:Sound;
		public var completeHandler:Function;
		public var priority:int;
		public var repeatCount:int;
		public var completeHandlerArguments:Array;
		public var callCompleteHandlerOnStop:Boolean;
		public var volume:Number;
		public var callCompleteHandlerOnPrempting:Boolean;
		public var startTime:Number = 0;
		
		public var soundChannel:SoundChannel;
		public var soundTransform:SoundTransform;
		public var isPaused:Boolean = false;
		public var pausedPosition:Number;
		public var isPlaying:Boolean = false;
		
		private var originalVolume:Number = 1;
		private var isVolumeChanged:Boolean = false;
		
		public function play():void{
			if(sound != null){
				soundChannel = sound.play(startTime,repeatCount);
				isPlaying = true;
				soundTransform = new SoundTransform(volume);
				soundChannel.soundTransform = soundTransform;
				soundChannel.addEventListener(Event.SOUND_COMPLETE,onSoundComplete);
				dispatchEvent(new Event(SOUND_PLAYBACK_STARTED));
			}
		}
		
		public function pause():void {
			if(isPlaying){
				pausedPosition = soundChannel.position;
				isPaused = true;
				isPlaying = false;
				soundChannel.removeEventListener(Event.SOUND_COMPLETE,onSoundComplete);
				soundChannel.stop();
				soundChannel = null;
				soundTransform = null;
				dispatchEvent(new Event(SOUND_PAUSED));
			}
		}
		
		public function resume():void {
			if(isPaused){
				soundChannel = sound.play(pausedPosition,repeatCount);
				isPlaying = true;
				soundTransform = new SoundTransform(volume);
				soundChannel.soundTransform = soundTransform;
				soundChannel.addEventListener(Event.SOUND_COMPLETE,onSoundComplete);
				dispatchEvent(new Event(SOUND_RESUMED));
			}
		}
		
		private function onSoundComplete(e:Event):void {
			isPlaying = false;
			soundChannel.removeEventListener(Event.SOUND_COMPLETE,onSoundComplete);
			if(completeHandler != null){
				if(completeHandlerArguments != null){
					completeHandler.apply(null,completeHandlerArguments);
				} else {
					completeHandler();
				}
			}
			dispatchEvent(new Event(SOUND_COMPLETE));
			soundChannel = null;
			soundTransform = null;
			
		}
		
		public function stop():void {
			if(isPlaying){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE,onSoundComplete);
				soundChannel.stop();
				isPlaying = false;
				soundChannel = null;
				soundTransform = null;
				if(callCompleteHandlerOnStop && completeHandler != null){
					if(completeHandlerArguments != null){
						completeHandler.apply(null,completeHandlerArguments);
					} else{
						completeHandler();
					}
				}
				dispatchEvent(new Event(SOUND_STOPPED));
			}
		}
		
		public function changeVolume(newVolume:Number):void {
			originalVolume = volume;
			isVolumeChanged = true;
			volume = newVolume;
			soundTransform = new SoundTransform(newVolume);
			if(isPlaying){
				soundChannel.soundTransform = soundTransform;
			}
			dispatchEvent(new Event(SOUND_VOLUME_CHANGED));
		}
		
		public function restoreVolume():void {
			if(isVolumeChanged){
				volume = originalVolume;
				isVolumeChanged = false;
				soundTransform = new SoundTransform(volume);
				if(isPlaying){
					soundChannel.soundTransform = soundTransform;
				}
				dispatchEvent(new Event(SOUND_VOLUME_RESTORED));
			}
		}
		
		public function dispose():void {
			if(soundChannel != null){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE,onSoundComplete);
			}
			sound = null;
			completeHandler = null;
			soundTransform = null;
			completeHandlerArguments = null;
		}
	}
	
}