﻿package com.callystro.sound {
	
	import flash.events.Event;
	import flash.media.Sound;
	import flash.media.SoundMixer;
	import flash.media.SoundTransform;
	import flash.media.SoundChannel;
	
	public class SoundManager {
		
		protected var soundInstanceContainer:SoundInstanceContainer;
		
		protected var backgroundMusicPlaying:Boolean = false;
		
		protected var backgroundMusicChannel:SoundChannel;
		
		protected var backgroundMusicVolume:Number = 0.3;
		
		public function SoundManager(soundInstanceContainer:SoundInstanceContainer){
			this.soundInstanceContainer = soundInstanceContainer;
		}
		
		public function play(soundInstance:SoundInstance):void {
			if(!SoundPlayer.allowMultipleInstanceOfSameSoundToPlayFlag){
				if(soundInstanceContainer.getSoundInstance(soundInstance.key) != null){
					return;
				}
			}
			soundInstanceContainer.addSoundInstance(soundInstance);
			soundInstance.addEventListener(SoundInstance.SOUND_COMPLETE,soundCompleteHandler);
			soundInstance.addEventListener(SoundInstance.SOUND_STOPPED,soundStoppedHandler);
			soundInstance.play();
		}
		
		protected function initialize():void {
			var allSoundInstances:Array;
			allSoundInstances = soundInstanceContainer.getAllSoundInstances();
			for(var i:int = 0;i < allSoundInstances.length; i++){
				allSoundInstances[i].addEventListener(SoundInstance.SOUND_COMPLETE,soundCompleteHandler);
				allSoundInstances[i].addEventListener(SoundInstance.SOUND_STOPPED,soundStoppedHandler);
			}
		}
		
		public function soundCompleteHandler(e:Event):void {
			e.target.removeEventListener(SoundInstance.SOUND_COMPLETE,soundCompleteHandler);
			e.target.removeEventListener(SoundInstance.SOUND_STOPPED,soundStoppedHandler);
			soundInstanceContainer.removeSoundInstance(e.target.key);
		}
		
		public function soundStoppedHandler(e:Event):void {
			e.target.removeEventListener(SoundInstance.SOUND_COMPLETE,soundCompleteHandler);
			e.target.removeEventListener(SoundInstance.SOUND_STOPPED,soundStoppedHandler);
			soundInstanceContainer.removeSoundInstance(e.target.key);
		}
		
		public function stop(key:String){
			var soundInstance:SoundInstance = soundInstanceContainer.getSoundInstance(key);
			if(soundInstance != null){
				soundInstance.stop();
			}
		}
		
		public function stopAll():void {
			var payingSoundInstances:Array;
			payingSoundInstances = soundInstanceContainer.getAllPlayingSoundInstances();
			for(var i:int = 0;i < payingSoundInstances.length; i++){
				payingSoundInstances[i].stop();
			}
		}
		
		public function pause(key:String){
			var soundInstance:SoundInstance = soundInstanceContainer.getSoundInstance(key);
			if(soundInstance != null){
				soundInstance.pause();
			}
		}
		
		public function pauseAll():void{
			var payingSoundInstances:Array;
			payingSoundInstances = soundInstanceContainer.getAllPlayingSoundInstances();
			for(var i:int = 0;i < payingSoundInstances.length; i++){
				payingSoundInstances[i].pause();
			}
		}
		
		public function resume(key:String):void {
			var soundInstance:SoundInstance = soundInstanceContainer.getSoundInstance(key);
			if(soundInstance != null){
				soundInstance.resume();
			}
		}
		
		public function resumeAll():void {
			var pausedSoundInstances:Array;
			pausedSoundInstances = soundInstanceContainer.getAllPausedSoundInstances();
			for(var i:int = 0;i < pausedSoundInstances.length; i++){
				pausedSoundInstances[i].resume();
			}
		}
		
		public function changeVolume(key:String,volume:Number):void{
			var soundInstance:SoundInstance = soundInstanceContainer.getSoundInstance(key);
			if(soundInstance != null){
				soundInstance.changeVolume(volume);
			}
		}
		
		public function restoreVolume(key:String):void{
			var soundInstance:SoundInstance = soundInstanceContainer.getSoundInstance(key);
			if(soundInstance != null){
				soundInstance.restoreVolume();
			}
		}
		
		public function playBackGroundMusic(sound:Sound,volume:Number):void
		{
			if(backgroundMusicChannel != null)
			{
				backgroundMusicChannel.stop();
				backgroundMusicChannel = null;
			}
			backgroundMusicVolume = volume;
			var sndTransform:SoundTransform = new SoundTransform(backgroundMusicVolume);
			backgroundMusicChannel = sound.play(0,int.MAX_VALUE,sndTransform);
			backgroundMusicPlaying = true;
		}
		
		public function dispose():void {
			var allSoundInstances:Array;
			allSoundInstances = soundInstanceContainer.getAllSoundInstances();
			for(var i:int = 0;i < allSoundInstances.length; i++){
				allSoundInstances[i].removeEventListener(SoundInstance.SOUND_COMPLETE,soundCompleteHandler);
				allSoundInstances[i].removeEventListener(SoundInstance.SOUND_STOPPED,soundStoppedHandler);
			}
			soundInstanceContainer = null;
		}
		
		public function stopBackGroundMusic():void
		{
			if(backgroundMusicChannel != null)
			{
				backgroundMusicChannel.stop();
				backgroundMusicChannel = null;
				backgroundMusicPlaying = false;
			}
		}
	
	}
	
}